function ASR1_main(varargin);
%This function contains the main algorithm for Automated Speech Recognition
%of spoken English digits using Dynamic Time Warping and 17 linear energy
%bands as a feature set.
%
%Can accept test data energy, test data sample rate, and test data bit rate
%or can accept a .mat file that has these saved named as: refDataE, fs &
%bits
%
%Created by Nigel Stuke for coursework in the class EEN 540 at the 
%University of Miami under the instruction of Dr. Scordilis
%Created on May 06, 2008
if length(varargin)==1
    load(varargin{1},'-mat');
else
    refDataE=varargin{1};
    fs=varargin{2};
    bits=varargin{3};
end;
    
    
numSetsRefData=size(refDataE,1);
[filename pathname]=uigetfile('*.wav','Please select the Test .wav file');
testDataLoc=cat(2,pathname, filename);
[testData,fsTest,bitsTest]=wavread(testDataLoc);
if (fsTest ~= fs)
    error('Error... Sampling Rate of Test Data Does Not Match Reference Data Sets');
end;
if (bitsTest ~= bits)
    error('Error... Bit Depth of Test Data Does Not Match Reference Data Sets');
end;
%Create window size of 20ms
windowSize=ceil(fs*20*10^-3);
win=window(@hamming,windowSize);
%take FFT with 50% overlap 
testDataFFT=spectrogram(testData,win,ceil(length(win)/2),33,fs);      
%Get Energy
testDataE=abs(testDataFFT).^2;
%normalize energy
totalEnergy=sum(sum(testDataE,1));
testDataE=testDataE./totalEnergy;
%Create Local Distance Matrix
%k is reference data set, m is reference digit
%p is frame reference of curr. test data loc.
%q is frame reference of curr. ref data loc.
for k=1:numSetsRefData
    for m=0:9
        clear currDist;
        for p=1:size(testDataE,2)
            for q=1:size(refDataE{k,m+1},2)
                currDist(p,q)=sum(abs(refDataE{k,m+1}(:,q)-testDataE(:,p)));
            end;
        end;
        localDists{k,m+1}=currDist;

    end;
end;
%Make Matrix of Accumulated Distances

for k=1:numSetsRefData
    for m=0:9
        clear accumDist;
        accumDist(1,1)=localDists{k,m+1}(1,1);
        for p=1:size(testDataE,2)
            for q=1:size(refDataE{k,m+1},2)
                if (p == 1 && q == 1)
                    %That Value Is an initial condition in the matrix
                elseif (p == 1)
                    accumDist(p,q)=accumDist(p,q-1)+localDists{k,m+1}(p,q); 
                elseif (q == 1)
                    accumDist(p,q)=accumDist(p-1,q)+localDists{k,m+1}(p,q);
                else
                    possDists=[accumDist(p,q-1) accumDist(p-1,q) accumDist(p-1,q-1)];
                    accumDist(p,q)=min(possDists)+localDists{k,m+1}(p,q);
                end;
            end;
        end;
        totalDists{k,m+1}=accumDist;
    end;
end;
%Create Matrix of final distances
for k=1:numSetsRefData
    for m=0:9
        finalDist(k,m+1)=totalDists{k,m+1}(size(totalDists{k,m+1},1),size(totalDists{k,m+1},2));
    end;
end;
[bestMatchRow,bestMatchCol] =find(finalDist==min(min(finalDist)));
if (length(bestMatchCol) == 1)
    disp('');
    disp(['The test sample was matched to be digit ' num2str(bestMatchCol - 1)]);
else
    %Add up all test data for the tied digits
    %Go with whichever one is lowest
    %Set all columns to a ridiculous number
    %This prevents it from picking a digit that wasn't tied
    colTotals=ones(1,10);
    colTotals=colTotals.*10^9;
    for k=1:length(bestMatchCol)
        colTotals(bestMatchCol(k))=sum(finalDist(:,bestMatchCol(k)));
    end;
    %Now it grabs the 1st matching index
    %If there is still multiple tied values... that's life, you can't
    %change it.. it takes a uneducated non-random guess of the first value
    [bestMatchRow,bestMatchCol]=find(colTotals==min(colTotals),1);
    disp('');
    disp(['The test sample was matched to be digit ' num2str(bestMatchCol - 1)]);
end;
